@@ -915,11 +915,15 @@ nautilus_mime_set_short_list_applications_for_file (NautilusFile      *file,
 		normal_short_list_ids = g_list_prepend (normal_short_list_ids, ((GnomeVFSMimeApplication *) p->data)->id);
 	}
 
+	gnome_vfs_mime_application_list_free (normal_short_list);
+
 	/* compute delta */
 
 	add_list = str_list_difference (applications, normal_short_list_ids);
 	remove_list = str_list_difference (normal_short_list_ids, applications);
 
+	eel_g_list_free_deep (normal_short_list_ids);
+
 	nautilus_file_set_metadata_list 
 		(file,
 		 NAUTILUS_METADATA_KEY_SHORT_LIST_APPLICATION_ADD,
@@ -931,9 +935,8 @@ nautilus_mime_set_short_list_applications_for_file (NautilusFile      *file,
 		 NAUTILUS_METADATA_SUBKEY_APPLICATION_ID,
 		 remove_list);
 
-	/* FIXME bugzilla.gnome.org 41269: 
-	 * need to free normal_short_list, normal_short_list_ids, add_list, remove_list 
-	 */
+	eel_g_list_free_deep (add_list);
+	eel_g_list_free_deep (remove_list);
 
 	return GNOME_VFS_OK;
 }
@@ -953,6 +956,7 @@ nautilus_mime_set_short_list_components_for_file (NautilusFile      *file,
 			      GNOME_VFS_ERROR_GENERIC);
 
 	/* get per-mime short list */
+
 	mime_type = nautilus_file_get_mime_type (file);
 	normal_short_list = gnome_vfs_mime_get_short_list_components (mime_type);
 	g_free (mime_type);
@@ -967,6 +971,9 @@ nautilus_mime_set_short_list_components_for_file (NautilusFile      *file,
 	add_list = str_list_difference (components, normal_short_list_ids);
 	remove_list = str_list_difference (normal_short_list_ids, components);
 
+	gnome_vfs_mime_component_list_free (normal_short_list);
+	g_list_free (normal_short_list_ids);
+
 	nautilus_file_set_metadata_list 
 		(file,
 		 NAUTILUS_METADATA_KEY_SHORT_LIST_COMPONENT_ADD,
@@ -978,9 +985,8 @@ nautilus_mime_set_short_list_components_for_file (NautilusFile      *file,
 		 NAUTILUS_METADATA_SUBKEY_COMPONENT_IID,
 		 remove_list);
 
-	/* FIXME bugzilla.gnome.org 41269: 
-	 * need to free normal_short_list, normal_short_list_ids, add_list, remove_list 
-	 */
+	eel_g_list_free_deep (add_list);
+	eel_g_list_free_deep (remove_list);
 
 	return GNOME_VFS_OK;
 }
@@ -1531,16 +1537,18 @@ server_matches_content_requirements (Bonobo_ServerInfo *server,
 }
 
 
+/* FIXME: do we actually need this it would seem to me that the
+ * test_only attribute handles this
+ */
 static char *nautilus_sort_criteria[] = {
         /* Prefer anything else over the loser view. */
-        "iid != 'OAFIID:nautilus_content_loser:95901458-c68b-43aa-aaca-870ced11062d'",
+        "iid != 'OAFIID:Nautilus_Content_Loser'",
         /* Prefer anything else over the sample view. */
-        "iid != 'OAFIID:nautilus_sample_content_view:45c746bc-7d64-4346-90d5-6410463b43ae'",
+        "iid != 'OAFIID:Nautilus_Sample_Content_View'",
 	/* Sort alphabetically */
 	"name",
-        NULL};
-
-
+        NULL
+};
 
 static GList *
 nautilus_do_component_query (const char        *mime_type, 
@@ -1590,10 +1598,7 @@ nautilus_do_component_query (const char        *mime_type,
 
                         if (ignore_content_mime_types || 
 			    server_matches_content_requirements (server, content_types, explicit_iids)) {
-                                /* Hack to suppress the Bonobo_Sample_Text component, since the Nautilus text
-                                 * view is a superset and it's confusing for the user to be presented with both
-                                 */
-                                if (server->iid != NULL && strcmp (server->iid, "OAFIID:Bonobo_Sample_Text") != 0) {
+                                if (server->iid != NULL) {
                                 	retval = g_list_prepend
                                         	(retval, 
 						 Bonobo_ServerInfo_duplicate (server));
